package unileipzig.isys.neuronviz.streamer;

import unileipzig.isys.neuronviz.kernel.DataStreamer;

/**
 * Class StandardInputStreamer
 * 
 */
public class StandardInputStreamer extends DataStreamer
{	
	public StandardInputStreamer()
	{
		setInputStream(System.in);
	}

}
