/***************************************************************************
 *   Copyright (C) 2005 by Dominic Schneider   *
 *   dominic@isyspc8   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qdatetime.h>
#include <qobject.h>
#include <qstring.h>

/** \brief Short class for logging char* strings to file named with date and time.
  * \author Dominic Schneider
  */
class FileLogger : public QObject
{
    Q_OBJECT

public:
    FileLogger(QString pf="");
    ~FileLogger();
    void setPrefix(QString pf) {prefix = pf;};
    void setLogging(bool log)  {this->log = log;}
    
private:
    QString prefix;
    QString filename;
    bool log;
    FILE *instream;

private slots:
    void openStream();  // opens the stream
    void writeChannelData(char *);   // writes the block as it gets it to file

};
