package unileipzig.isys.neuronviz.streamer;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import unileipzig.isys.neuronviz.kernel.DataStreamer;

public class FileInputStreamer extends DataStreamer
{
	public FileInputStreamer(String fileName) throws FileNotFoundException
	{
		setInputStream(new FileInputStream(fileName));
	}
}
